UIPanelWindows["ZonePopFrame"] =		{ area = "left",	pushable = 10 };

-- Constants
ZONEPOP_ITEMDISPLAYCOUNT = 21;
ZONEPOP_FRAME_LISTITEM_HEIGHT = 16;
ZONEPOP_COL_ZONE = 1;
ZONEPOP_COL_POP = 2;
ZONEPOP_UPDATEINTERVAL = 20;

-- Variables
ZonePop_ZoneCount = 0;
ZonePop_ZoneList = {};
ZonePop_TotalPop = 0;
ZonePop_SortCol = ZONEPOP_COL_ZONE;
ZonePop_SortDir = true;
ZonePop_CurWhoZone = 0;
ZonePop_WhoAllZones = 0;
ZonePop_LastUpdate = 0;

-- Functions
function ToggleZonePop()
	if ( ZonePopFrame:IsVisible() ) then
		HideUIPanel(ZonePopFrame);
	else
		ShowUIPanel(ZonePopFrame);
	end
end

function ZonePop_OnLoad()
	Cosmos_RegisterButton ( ZONEPOP_BUTTON_TITLE, ZONEPOP_BUTTON_SUBTITLE, ZONEPOP_BUTTON_TIP, "Interface\\FriendsFrame\\FriendsFrameScrollIcon", ToggleZonePop);
	Cosmos_RegisterChatWatch ( "ZONEPOP", {"SYSTEM"}, function (type, info, arg1, arg2, arg3, arg4) return ZonePop_ProcessSystemMessage(arg1); end );
    this:RegisterEvent("WHO_LIST_UPDATE");
    ZonePop_LoadZoneList();
    ZonePop_UpdateList();
end

function ZonePop_LoadZoneList()
    ZonePop_ZoneCount = 0;
    ZonePop_LoadContinents(GetMapContinents());
    ZonePop_SortList();
end

function ZonePop_LoadContinents(...)
	for i=1, arg.n, 1 do
		ZonePop_LoadZones(GetMapZones(i));
	end
end

function ZonePop_LoadZones(...)
	local info;
	for i=1, arg.n, 1 do
		ZonePop_ZoneCount = ZonePop_ZoneCount + 1;
      	ZonePop_ZoneList[ZonePop_ZoneCount] = { name = arg[i], pop = 0 };
	end
end

function ZonePop_CompareZones(zone1, zone2)
    if (ZonePop_SortCol == ZONEPOP_COL_ZONE) then
        if (ZonePop_SortDir) then
            return (zone1.name < zone2.name);
        else
            return (zone1.name > zone2.name);
        end
    elseif (ZonePop_SortCol == ZONEPOP_COL_POP) then
        if (ZonePop_SortDir) then
            return (zone1.pop < zone2.pop);
        else
            return (zone1.pop > zone2.pop);
        end
    end
    
    return 0;
end

function ZonePop_SortList()
    table.sort(ZonePop_ZoneList, ZonePop_CompareZones);
end

function ZonePop_OnShow()
    PlaySound("igMainMenuOpen");
    ZonePop_UpdateAll();
end

function ZonePop_OnHide()
    PlaySound("igMainMenuClose");
end

function ZonePop_OnUpdateFrame()
    local curTime = GetTime();
    local elapsed = curTime - ZonePop_LastUpdate;
    if (elapsed < ZONEPOP_UPDATEINTERVAL) then 
        return;
    end
    
    ZonePop_UpdateAll();
end

function ZonePop_OnEvent(event)
    if (event == "WHO_LIST_UPDATE") then
        ZonePop_OnWhoReturned();
   	end
end

function ZonePop_OnColumnClick(col)
    if (ZonePop_SortCol == col) then
        ZonePop_SortDir = not ZonePop_SortDir;
    else
        ZonePop_SortCol = col;
        ZonePop_SortDir = true;
    end
    
    ZonePop_SortList();
    ZonePop_UpdateList();
end

function ZonePop_OnItemClick()
    local zoneId = getglobal("ZonePopFrameListItem"..this:GetID()).zoneId;
    ZonePopFrame.selectedZone = ZonePop_ZoneList[zoneId].name;
    ZonePop_UpdateList();
end

function ZonePop_UpdateList()
    local numZones = ZonePop_ZoneCount;
    local zoneId;
    local listItem;
    local scrollOffset = FauxScrollFrame_GetOffset(ZonePopListScrollFrame);
     
    for i=1, ZONEPOP_ITEMDISPLAYCOUNT, 1 do
        zoneId = i + scrollOffset;
		listItem = getglobal("ZonePopFrameListItem"..i);
        listItem.zoneId = zoneId;
		getglobal("ZonePopFrameListItem"..i.."Zone"):SetText(ZonePop_ZoneList[zoneId].name);
		getglobal("ZonePopFrameListItem"..i.."Population"):SetText(ZonePop_ZoneList[zoneId].pop);
        
        if ( ZonePopFrame.selectedZone == ZonePop_ZoneList[zoneId].name ) then
			listItem:LockHighlight();
		else
			listItem:UnlockHighlight();
		end
        
        if ( zoneId > numZones ) then
			listItem:Hide();
		else
			listItem:Show();
		end
	end
    
    FauxScrollFrame_Update(ZonePopListScrollFrame, numZones, ZONEPOP_ITEMDISPLAYCOUNT, ZONEPOP_FRAME_LISTITEM_HEIGHT);

    ZonePopTotalText:SetText(ZonePop_TotalPop);
end

function ZonePop_OnWhoReturned()
    if (ZonePop_CurWhoZone > 0) then
        local numWhos, totalCount = GetNumWhoResults();
        
        if (ZonePop_CurWhoZone <= ZonePop_ZoneCount) then
            ZonePop_ZoneList[ZonePop_CurWhoZone].pop = totalCount;
        else
            ZonePop_TotalPop = totalCount;
        end
        
        if (ZonePop_WhoAllZones == 1) then
            if (ZonePop_CurWhoZone <= ZonePop_ZoneCount) then
                ZonePop_UpdatePopulation(ZonePop_CurWhoZone + 1);
            else
                ZonePop_CurWhoZone = 0;
                ZonePop_WhoAllZones = 0;
                ZonePop_OnPopulationUpdateComplete();
            end
        end
    end
end

function ZonePop_ProcessSystemMessage(msg)
    if (ZonePop_CurWhoZone == 0) then
        return 1; -- Show
  	end
    
    if (string.find(msg, WHO_NUM_RESULTS) or string.find(msg, WHO_NUM_RESULTS_P1)) then
        ZonePop_OnWhoReturned();
    end
    return 0; -- Hide
end

function ZonePop_UpdatePopulation(zoneId)
    FriendsFrame_IgnoreNextWho();
    ZonePop_CurWhoZone = zoneId;
    
    if (zoneId > ZonePop_ZoneCount) then
        SendWho("");
   	else
        SendWho(ZonePop_ZoneList[zoneId].name);
    end
end

function ZonePop_UpdatePopulations()
    ZonePop_WhoAllZones = 1;
    ZonePop_UpdatePopulation(1);
end

function ZonePop_OnPopulationUpdateComplete()
    if (ZonePop_SortCol == ZONEPOP_COL_POP) then
        ZonePop_SortList();
    end
    ZonePop_UpdateList();
end

function ZonePop_UpdateAll()
    ZonePop_UpdatePopulations();
    ZonePop_LastUpdate = GetTime();
end

function ZonePop_DebugMessage(msg)
    local info = ChatTypeInfo["SYSTEM"];
    ChatFrame1:AddMessage(msg, info.r, info.g, info.b, info.id);
    --PlaySound("TellMessage");
end
